@@ -1,7 +1,7 @@
 """
 PySCeS - Python Simulator for Cellular Systems (http://pysces.sourceforge.net)
 
-Copyright (C) 2004-2012 B.G. Olivier, J.M. Rohwer, J.-H.S Hofmeyr all rights reserved,
+Copyright (C) 2004-2014 B.G. Olivier, J.M. Rohwer, J.-H.S Hofmeyr all rights reserved,
 
 Brett G. Olivier (bgoli@users.sourceforge.net)
 Triple-J Group for Molecular Cell Physiology
@@ -15,15 +15,14 @@ NO WARRANTY IS EXPRESSED OR IMPLIED.  USE AT YOUR OWN RISK.
 Brett G. Olivier
 """
 
-from version import __version__
+from __future__ import division, print_function, absolute_import
+from .version import __version__
 
 import os
 import math, operator
 import numpy
-##  import scipy
-# CurrentDirectory = os.path.dirname(os.path.abspath(os.sys.argv[0]))
 
-from InfixParser import MyInfixParser
+from .InfixParser import MyInfixParser
 
 InfixParser = MyInfixParser()
 InfixParser.buildlexer()
@@ -99,16 +98,16 @@ class Compartment(NewCoreBase):
 
     def setSize(self, size, dim):
         self.size = size
-        assert dim in [0,1,2,3], '\nOkeee! %s dimensions?' % dim
+        assert dim in [0,1,2,3], '\nOkeee! {0:d} dimensions?'.format(dim)
         self.dimensions = dim
 
     def setArea(self, area=None):
         if area == None and self.dimensions == 2:
             self.area = self.size
-            if self.__DEBUG__: print 'Setting reactive area to size for 2D compartment %s' % self.name
+            if self.__DEBUG__: print('Setting reactive area to size for 2D compartment {0:s}'.format(self.name) )
         elif area == None and self.dimensions == 3:
             self.area = (113.09733552923255*self.size**2.0)**(0.33333333333333331)
-            if self.__DEBUG__: print 'Setting reactive area to surface area for 3D compartment %s (assuming a sphere geometry)' % self.name
+            if self.__DEBUG__: print('Setting reactive area to surface area for 3D compartment {0:s} (assuming a sphere geometry)'.format(self.name) )
         self.area = area
 
     def hasReactions(self):
@@ -121,15 +120,15 @@ class Compartment(NewCoreBase):
         if reaction.name not in self.hasReactions():
             self.reactions.append(reaction)
             self.__setattr__(reaction.name, reaction)
-            if self.__DEBUG__: print 'Adding reaction %s' % reaction.name
+            if self.__DEBUG__: print('Adding reaction {0:s}'.format(reaction.name) )
 
     def addSpecies(self, species):
         if species.name not in self.hasSpecies():
             self.species.append(species)
             self.__setattr__(species.name, species)
-            if self.__DEBUG__: print 'Adding species %s' % species.name
+            if self.__DEBUG__: print('Adding species {0:s}'.format(species.name) )
         else:
-            if self.__DEBUG__: print 'Species %s already added' % species.name
+            if self.__DEBUG__: print('Species {0:s} already added'.format(species.name) )
 
     def getDimensions(self):
         return self.dimensions
@@ -168,7 +167,7 @@ class BaseUnit(NewCoreBase):
 
     def __init__(self, name, type):
         self.name = name
-        assert type in self._types, '\nType must be one of: %s' % str(self._types)
+        assert type in self._types, '\nType must be one of: {0:s}'.format( str(self._types) )
         self.type = type
 
     def __call__(self):
@@ -221,7 +220,7 @@ class CompoundUnit(NewCoreBase):
     def addUnit(self, unit):
         self.units.append(unit)
         if not self._HAS_USERNAME:
-            self.name = '%s%s' % (self.name, unit.getName())
+            self.name = '{0:s}{1}'.format(self.name, unit.getName())
 
     def getUnits(self):
         return self.units
@@ -308,7 +307,7 @@ class SpeciesAssignmentRule(Species):
         self.formula = formula
         InfixParser.setNameStr('self.', '()')
         InfixParser.parse(formula)
-        self.code_string = 'self.value=%s' % InfixParser.output
+        self.code_string = 'self.value={0}'.format(InfixParser.output)
         self._names = InfixParser.names
         self._functions = InfixParser.functions
         self.xcode = compile(self.code_string, '<string>', 'exec')
@@ -346,7 +345,7 @@ class Function(NewCoreBase):
         InfixParser.SymbolReplacements = {'_TIME_':'_TIME_()'}
         InfixParser.parse(formula)
         self._names = InfixParser.names
-        self.code_string = 'self.value=%s' % InfixParser.output
+        self.code_string = 'self.value={0}'.format(InfixParser.output)
         self.xcode = compile(self.code_string, '<string>', 'exec')
 
 class Reaction(NewCoreBase):
@@ -401,7 +400,7 @@ class Reaction(NewCoreBase):
         InfixParser.parse(formula)
         self._names = InfixParser.names
         self._functions = InfixParser.functions
-        self.code_string = 'self.rate=%s' % InfixParser.output
+        self.code_string = 'self.rate={0}'.format(InfixParser.output)
         self.xcode = compile(self.code_string, '<string>', 'exec')
 
     def addParameter(self, par):
@@ -477,7 +476,7 @@ class AssignmentRule(Parameter):
         self.formula = formula
         InfixParser.setNameStr('self.', '()')
         InfixParser.parse(formula)
-        self.code_string = 'self.value=%s' % InfixParser.output
+        self.code_string = 'self.value={0}'.format(InfixParser.output)
         self._names = InfixParser.names
         self._functions = InfixParser.functions
         self.xcode = compile(self.code_string, '<string>', 'exec')
@@ -508,10 +507,10 @@ class RateRule(NewCoreBase):
         self.formula = formula.replace('()','')
         InfixParser.setNameStr('self.', '()')
         InfixParser.parse(self.formula)
-        self.code_string = 'self.rate=%s' % InfixParser.output
+        self.code_string = 'self.rate={0}'.format(InfixParser.output)
         self._names = InfixParser.names
         self._functions = InfixParser.functions
-        self.xcode = compile(self.code_string, 'RateRule: %s' % self.name, 'exec')
+        self.xcode = compile(self.code_string, 'RateRule: {0:s}'.format(self.name), 'exec')
 
     def getFormula(self):
         return self.formula
@@ -549,26 +548,26 @@ class ODE(NewCoreBase):
         self.coefficients.append(coefficient)
         if coefficient > 0.0:
             if coefficient == 1.0:
-                term = '+self.%s() ' % (reaction.name)
-                aterm = '+(%s) ' % (reaction.code_string.replace('self.rate=',''))
-                fterm = '+%s' % (reaction.name)
-                afterm = '+ (%s) ' % (reaction.formula)
+                term = '+self.{0:s}() '.format(reaction.name)
+                aterm = '+({0}) '.format(reaction.code_string.replace('self.rate=',''))
+                fterm = '+{0:s} '.format(reaction.name)
+                afterm = '+ ({0}) '.format(reaction.formula)
             else:
-                term = '+%g*self.%s() ' % (abs(coefficient), reaction.name)
-                aterm = '+%g*(%s) ' % (abs(coefficient), reaction.code_string.replace('self.rate=',''))
-                fterm = '+%g*%s' % (abs(coefficient), reaction.name)
-                afterm = '+ %g*(%s) ' % (abs(coefficient), reaction.formula)
+                term = '+{0}*self.{1}() '.format(abs(coefficient), reaction.name)
+                aterm = '+{0}*({1}) '.format(abs(coefficient), reaction.code_string.replace('self.rate=',''))
+                fterm = '+{0}*{1}'.format(abs(coefficient), reaction.name)
+                afterm = '+ {0}*({1}) '.format(abs(coefficient), reaction.formula)
         else:
             if coefficient == -1.0:
-                term = '-self.%s() ' % (reaction.name)
-                aterm = '-(%s) ' % (reaction.code_string.replace('self.rate=',''))
-                fterm = '-%s' % (reaction.name)
-                afterm = '- (%s) ' % (reaction.formula)
+                term = '-self.{0:s}() '.format(reaction.name)
+                aterm = '-({0}) '.format(reaction.code_string.replace('self.rate=',''))
+                fterm = '-{0:s} '.format(reaction.name)
+                afterm = '- ({0}) '.format(reaction.formula)
             else:
-                term = '-%g*self.%s() ' % (abs(coefficient), reaction.name)
-                aterm = '-%g*(%s) ' % (abs(coefficient), reaction.code_string.replace('self.rate=',''))
-                fterm = '-%g*%s' % (abs(coefficient), reaction.name)
-                afterm = '- %g*(%s) ' % (abs(coefficient), reaction.formula)
+                term = '-{0}*self.{1}() '.format(abs(coefficient), reaction.name)
+                aterm = '-{0}*({1}) '.format(abs(coefficient), reaction.code_string.replace('self.rate=',''))
+                fterm = '-{0}*{1}'.format(abs(coefficient), reaction.name)
+                afterm = '- {0}*({1}) '.format(abs(coefficient), reaction.formula)
         self.ode_terms.append(term)
         self.code_string += term
         self.code_string_alt += aterm
@@ -642,9 +641,9 @@ class StructMatrix(NewCoreBase):
         assert self.row != None, "\nI need row labels"
         try:
             return self.array.take([self.row.index(l) for l in args], axis=0)
-        except Exception, ex:
-            print ex
-            print "\nValid row labels are: %s" % self.row
+        except Exception as ex:
+            print(ex)
+            print("\nValid row labels are: {0}".format(self.row) )
             return None
 
     def getColsByName(self, *args):
@@ -652,9 +651,9 @@ class StructMatrix(NewCoreBase):
         assert self.col != None, "\nI need column labels"
         try:
             return self.array.take([self.col.index(l) for l in args], axis=1)
-        except Exception, ex:
-            print ex
-            print "Valid column labels are: %s" % self.col
+        except Exception as  ex:
+            print(ex)
+            print("Valid column labels are: {0}".format(self.col) )
             return None
 
     def getLabels(self, axis='all'):
@@ -670,23 +669,23 @@ class StructMatrix(NewCoreBase):
         else: return self.ridx, self.cidx
 
     def getByIdx(self, row, col):
-        assert row in self.ridx, '\n%s is an invalid index' % row
-        assert col in self.cidx, '\n%s is an invalid index' % col
+        assert row in self.ridx, '\n{0} is an invalid index'.format(row)
+        assert col in self.cidx, '\n{0} is an invalid index'.format(col)
         return self.array[row, col]
 
     def getByName(self, row, col):
-        assert row in self.row, '\n%s is an invalid name' % row
-        assert col in self.col, '\n%s is an invalid name' % col
+        assert row in self.row, '\n{0} is an invalid name'.format(row)
+        assert col in self.col, '\n{0} is an invalid name'.format(col)
         return self.array[self.row.index(row), self.col.index(col)]
 
     def setByIdx(self, row, col, val):
-        assert row in self.ridx, '\n%s is an invalid index' % row
-        assert col in self.cidx, '\n%s is an invalid index' % col
+        assert row in self.ridx, '\n{0} is an invalid index'.format(row)
+        assert col in self.cidx, '\n{0} is an invalid index'.format(col)
         self.array[row, col] = val
 
     def setByName(self, row, col, val):
-        assert row in self.row, '\n%s is an invalid name' % row
-        assert col in self.col, '\n%s is an invalid name' % col
+        assert row in self.row, '\n{0} is an invalid name'.format(row)
+        assert col in self.col, '\n{0} is an invalid name'.format(col)
         self.array[self.row.index(row), self.col.index(col)] = val
 
     def shape(self):
@@ -701,7 +700,7 @@ class EventAssignment(NumberBase):
 
     def __call__(self):
         self.variable.value = self.value
-        if self.__DEBUG__: print '\tAssigning %s = %s' % (self.variable.name, self.value)
+        if self.__DEBUG__: print('\tAssigning {0:s} = {1}'.format(self.variable.name, self.value))
         return True
 
     def __init__(self, name='None'):
@@ -716,7 +715,7 @@ class EventAssignment(NumberBase):
         ##  InfixParser.SymbolReplacements = {'_TIME_':'_TIME_()'}
         InfixParser.parse(formula)
         self._names = InfixParser.names
-        self.code_string = 'self.value=%s' % InfixParser.output
+        self.code_string = 'self.value={0}'.format(InfixParser.output)
         self.xcode = compile(self.code_string, '<string>', 'exec')
         if self.__DEBUG__: '\t', self.name, self.code_string
 
@@ -757,11 +756,11 @@ class Event(NewCoreBase):
             self.state0 = self.state
             self._need_action = True
             self._ASS_TIME_ = self._TIME_() + self.delay
-            if self.__DEBUG__: print 'event %s is evaluating at %s' % (self.name, time)
+            if self.__DEBUG__: print('event {0:s} is evaluating at {1}'.format(self.name, time))
         if self._need_action and self._TIME_() >= self._ASS_TIME_:
             for ass in self.assignments:
                 ass()
-            if self.__DEBUG__: print 'event %s is assigning at %s (delay=%s)' % (self.name, time, self.delay)
+            if self.__DEBUG__: print('event {0:s} is assigning at {1} (delay={2})'.format(self.name, time, self.delay))
             self._need_action = False
 
     def setTrigger(self, formula, delay=0.0):
@@ -774,7 +773,7 @@ class Event(NewCoreBase):
             ##  self.formula = formula.replace(self._time_symbol, '_TIME_')
         InfixParser.parse(formula)
         self._names = InfixParser.names
-        self.code_string = 'self.state=%s' % InfixParser.output
+        self.code_string = 'self.state={0}'.format(InfixParser.output)
         if self._time_symbol != None:
             InfixParser.setNameStr('', '')
             InfixParser.SymbolReplacements = {self._time_symbol : '_TIME_'}
@@ -812,22 +811,21 @@ class PieceWise(NewCoreBase):
     def __init__(self, pwd):
         pwd = pwd.copy()
         if pwd['other'] != None:
-            other = 'self.value = %s' % pwd.pop('other')
+            other = 'self.value = {0}'.format( pwd.pop('other') )
         else:
             other = 'pass'
             pwd.pop('other')
         InfixParser.setNameStr('self.', '')
         InfixParser.SymbolReplacements = {'_TIME_':'_TIME_()'}
         self._names = []
-        if len(pwd.keys()) == 1:
+        if len(list(pwd)) == 1:
             formula = pwd[0][0]
             InfixParser.parse(formula)
             for n in InfixParser.names:
                 if n not in self._names and n != '_TIME_()':
                     self._names.append(n)
             formula = InfixParser.output
-            self.code_string = 'if %s:\n    self.value = %s\nelse:\n    %s' %\
-            (formula, pwd[0][1], other)
+            self.code_string = 'if {0}:\n    self.value = {1}\nelse:\n    {2}'.format(formula, pwd[0][1], other)
             self.formula = self.code_string.replace('self.','')
         else:
             formula = pwd[0][0]
@@ -837,7 +835,7 @@ class PieceWise(NewCoreBase):
                     self._names.append(n)
 
             formula = InfixParser.output
-            self.code_string = 'if %s:\n    self.value = %s\n' % (formula, pwd[0][1])
+            self.code_string = 'if {0}:\n    self.value = {1}\n'.format(formula, pwd[0][1])
             pwd.pop(0)
             for p in pwd:
                 formula = pwd[p][0]
@@ -847,8 +845,8 @@ class PieceWise(NewCoreBase):
                     if n not in self._names and n != '_TIME_()':
                         self._names.append(n)
                 formula = InfixParser.output
-                self.code_string += 'elif %s:\n    self.value = %s\n' % (formula, pwd[p][1])
-            self.code_string += 'else:\n    %s' % other
+                self.code_string += 'elif {0}:\n    self.value = {1}\n'.format(formula, pwd[p][1])
+            self.code_string += 'else:\n    {0}'.format(other)
             self.formula = self.code_string.replace('self.','')
         self.xcode = compile(self.code_string, 'PieceWise','exec')
 
@@ -922,7 +920,7 @@ class NewCore(NewCoreBase):
         self.__model__ = model
         self.__InitDict__ = model.__InitDict__
         if not iValues:
-            if self.__DEBUG__: print self.__InitDict__
+            if self.__DEBUG__: print(self.__InitDict__)
             for k in self.__InitDict__.keys():
                 self.__InitDict__[k] = getattr(self.__model__, k)
             for c in model.__compartments__:
@@ -950,7 +948,7 @@ class NewCore(NewCoreBase):
             self.__functions__ = model.__functions__
         except:
             self.__functions__ = {}
-            if self.__DEBUG__: print 'No functions'
+            if self.__DEBUG__: print('No functions')
         self.functions = []
         self.addFunctions()
         self.addReactions()
@@ -967,27 +965,13 @@ class NewCore(NewCoreBase):
         ##  # get rid of _TIME_ in not intited
         ##  if '_TIME_' in self.__not_inited__:
             ##  self.__not_inited__.pop(self.__not_inited__.index('_TIME_'))
-        assert len(self.__not_inited__) < 1, "\nERROR: Uninitialised parameters: %s" % self.__not_inited__
+        assert len(self.__not_inited__) < 1, "\nERROR: Uninitialised parameters: {0}".format(self.__not_inited__)
 
     def __cleanString__(self,s):
         s = s.lstrip()
         s = s.rstrip()
         return s
 
-    ##  def parseForcingFunctions(self):
-        ##  self.__rules__ = {}
-        ##  try:
-            ##  ff = self.__function_forced_str__.split('\n')
-            ##  for f in ff:
-                ##  if f != '':
-                    ##  f =  f.split('=')
-                    ##  f[0] = f[0].replace('self.','')
-                    ##  f[1] = f[1].replace('self.','')
-                    ##  self.__rules__.setdefault(self.__cleanString__(f[0]), self.__cleanString__(f[1]))
-        ##  except Exception, ex:
-            ##  pass
-            # print 'No forcing functions (%s).' % ex
-
     def setDescription(self, txt):
         self.description = str(txt)
 
@@ -997,7 +981,7 @@ class NewCore(NewCoreBase):
     def setGlobalUnits(self, **kwargs):
         for un in kwargs.keys():
             self.__uDict__[un] = (kwargs[un][0], kwargs[un][1])
-            if self.__DEBUG__: print "Modified \"%s\" to be %i*%s*10**%i" % (un, kwargs[un][0], un, kwargs[un][1])
+            if self.__DEBUG__: print("Modified \"{0}\" to be {1}*{0}*10**{2}".format(un, kwargs[un][0], kwargs[un][1]))
 
     def getGlobalUnits(self):
         return self.__uDict__
@@ -1006,7 +990,7 @@ class NewCore(NewCoreBase):
         if not update:
             self.piecewise_functions = []
             for pw in self.__piecewises__.keys():
-                if self.__DEBUG__: print 'Info: adding piecewise function:%s' % pw
+                if self.__DEBUG__: print('Info: adding piecewise function:{0}'.format(pw) )
                 P = PieceWise(self.__piecewises__[pw])
                 P.setName(pw)
                 P.__setattr__('_TIME_', self.__getattribute__('_TIME_'))
@@ -1030,7 +1014,7 @@ class NewCore(NewCoreBase):
         self.compartments = []
         for C in self.__compartments__:
             c2 = self.__compartments__[C]
-            if self.__DEBUG__: print 'Adding compartment %s' % c2['name']
+            if self.__DEBUG__: print('Adding compartment {0}'.format(c2['name']))
             self.addOneCompartment(c2['name'], c2['size'], c2['dimensions'],
                             compartment=c2['compartment'], area=None)
 
@@ -1051,14 +1035,12 @@ class NewCore(NewCoreBase):
         self.species = []
         self.species_variable = []
         for s in self.__sDict__:
-            ##  print s
-            ##  print self.__sDict__[s]
             name = self.__sDict__[s]['name']
-            if self.__InitDict__.has_key(s):
+            if s in self.__InitDict__:
                 val = self.__InitDict__[s]
             else:
                 val = 0.0
-            ##  print val
+            
             fix = self.__sDict__[s]['fixed']
             if self.__sDict__[s]['compartment'] != None:
                 comp = self.__getattribute__(self.__sDict__[s]['compartment'])
@@ -1066,7 +1048,7 @@ class NewCore(NewCoreBase):
                 comp = None
             amount = self.__sDict__[s]['isamount']
             fullName = None
-            if self.__sDict__[s].has_key('fullName'):
+            if 'fullName' in self.__sDict__[s]:
                 fullName = self.__sDict__[s]['fullName']
             self.addOneSpecies(name, val, fix=fix, comp=comp, amount=amount, fullName=fullName)
 
@@ -1094,7 +1076,7 @@ class NewCore(NewCoreBase):
             r.setCompartment(C)
             C.addReaction(r)
         fullName = None
-        if rDict.has_key('fullName'):
+        if 'fullName' in rDict:
             r.setAnnotation('sbml_name', rDict['fullName'])
 
         # TODO: make better
@@ -1106,9 +1088,9 @@ class NewCore(NewCoreBase):
             for func in r._functions:
                 try:
                     r.addFunction(self.__getattribute__(func))
-                except Exception, ex:
-                    print ex
-                    print '\nHave you added the function objects yet (addFunctions())'
+                except Exception as ex:
+                    print(ex)
+                    print('\nHave you added the function objects yet (addFunctions())')
 
         #fxnames = self.hasFixedSpecies()
         processed_parameter = []
@@ -1116,14 +1098,14 @@ class NewCore(NewCoreBase):
         for p in rDict['Params']:
             p = p.replace('self.','')
             if p not in self.hasGlobalParameters() and not (p in self.hasFixedSpecies() or p in self.__compartments__):
-                if self.__DEBUG__: print "Adding parameter %s from networkdict" % p
+                if self.__DEBUG__: print("Adding parameter {0} from networkdict".format(p))
                 self.addParameter(p)
                 par = self.__getattribute__(p)
                 par.setAssociation(r)
                 r.addParameter(par)
                 processed_parameter.append(p)
             elif not (p in self.hasFixedSpecies() or p in self.__compartments__):
-                if self.__DEBUG__: print "Updating parameter %s from networkdict" % p
+                if self.__DEBUG__: print("Updating parameter {0} from networkdict".format(p))
                 pidx = self.hasGlobalParameters().index(p)
                 self.global_parameters[pidx].setAssociation(r)
                 r.addParameter(self.global_parameters[pidx])
@@ -1146,7 +1128,7 @@ class NewCore(NewCoreBase):
                 #setattr(r, C.name, C)
                 processed_parameter.append(p)
             elif p not in processed_parameter and p not in self.hasGlobalParameters() and p not in self.hasSpecies():
-                if self.__DEBUG__: print "Adding parameter %s from global" % p
+                if self.__DEBUG__: print("Adding parameter {0} from global".format(p) )
                 self.addParameter(p)
                 par = self.__getattribute__(p)
                 par.setAssociation(r)
@@ -1154,7 +1136,7 @@ class NewCore(NewCoreBase):
                 processed_parameter.append(p)
 
             elif p not in processed_parameter and p not in self.hasSpecies():
-                if self.__DEBUG__: print "Updating parameter %s from global" % p
+                if self.__DEBUG__: print("Updating parameter {0} from global".format(p) )
                 pidx = self.hasGlobalParameters().index(p)
                 self.global_parameters[pidx].setAssociation(r)
                 r.addParameter(self.global_parameters[pidx])
@@ -1164,8 +1146,8 @@ class NewCore(NewCoreBase):
         self.reactions.append(r)
 
     def addParameter(self, name):
-        if not self.__piecewises__.has_key(name):
-            if self.__InitDict__.has_key(name):
+        if not name in self.__piecewises__:
+            if name in self.__InitDict__:
                 par = Parameter(name, self.__InitDict__[name])
             else:
                 par = Parameter(name, 0.0)
@@ -1180,7 +1162,7 @@ class NewCore(NewCoreBase):
         non_parameters = self.hasGlobalParameters()+self.hasSpecies()+self.hasFixedSpecies()
         for k in self.__InitDict__.keys():
             if k not in non_parameters:
-                if self.__DEBUG__: print 'Adding new parameter:', k
+                if self.__DEBUG__: print( 'Adding new parameter:', k)
                 self.addParameter(k)
 
     def replaceParameterWithRule(self, ar):
@@ -1234,7 +1216,7 @@ class NewCore(NewCoreBase):
         for p in self.global_parameters + self.species:
             #print p.name
             if p.name in aps:
-                if self.__DEBUG__: print 'Assigning: %s = %s' % (p.name, self.__rules__[p.name]['formula'])
+                if self.__DEBUG__: print('Assigning: {0:s} = {1}'.format(p.name, self.__rules__[p.name]['formula']))
                 p2 = None
                 # TODO: make better
                 if p.name in self.hasGlobalParameters():
@@ -1284,7 +1266,7 @@ class NewCore(NewCoreBase):
             # TODO investgiate this as it is problematic, the rate rule
             # is not a model property as such more an ODE property
             ##  self.__setattr__(rrobj.name, rrobj)
-            if self.__DEBUG__: print 'Adding RateRule %s with formula: %s' % (rrobj.name, rrobj.formula)
+            if self.__DEBUG__: print( 'Adding RateRule {0} with formula: {1}'.format(rrobj.name, rrobj.formula))
 
     def addOneEvent(self, e):
         """Add a single event using an event dictionary """
@@ -1337,7 +1319,7 @@ class NewCore(NewCoreBase):
                         reac.addProduct(self.__getattribute__(reag[0].replace('self.','')))
                         self.__getattribute__(reag[0].replace('self.','')).setProduct(self.__getattribute__(reac.name))
                     reac.multistoich.append((reag[0].replace('self.',''), reag[1]))
-                    if reac.stoichiometry.has_key(reag[0].replace('self.','')):
+                    if reag[0].replace('self.','') in reac.stoichiometry:
                         reac.multistoich_enabled = True
                     reac.stoichiometry.setdefault(reag[0].replace('self.',''), reag[1])
             for mod in self.__nDict__[reac.name]['Modifiers']:
@@ -1371,18 +1353,18 @@ class NewCore(NewCoreBase):
         for varspec in self.stoichiometric_matrix.row:
             if self.struct != None:
                 if varspec not in self.struct.Nr.row:
-                    if self.__DEBUG__: print 'Creating dependent ODE_%s' % varspec
+                    if self.__DEBUG__: print('Creating dependent ODE_{0}'.format(varspec) )
                     ode = ODE(self.get(varspec), independent=False)
                 else:
-                    if self.__DEBUG__: print 'Creating independent ODE_%s' % varspec
+                    if self.__DEBUG__: print('Creating independent ODE_{0}'.format(varspec) )
                     ode = ODE(self.get(varspec), independent=True)
             else:
-                if self.__DEBUG__: print 'Creating independent* ODE_%s (*assumed - no structural information available)' % varspec
+                if self.__DEBUG__: print( 'Creating independent* ODE_{0} (*assumed - no structural information available)'.format(varspec) )
                 ode = ODE(self.get(varspec), independent=True)
             mrow = self.stoichiometric_matrix.getRowsByName(varspec)
             for e in range(len(mrow[0])):
                 if mrow[0,e] != 0.0:
-                    print 'Adding term: %s*%s' % (mrow[0,e], self.stoichiometric_matrix.col[e])
+                    print('Adding term: {0}*{1}'.format(mrow[0,e], self.stoichiometric_matrix.col[e]))
                     ode.addReaction(self.get(self.stoichiometric_matrix.col[e]), mrow[0,e])
             self.__setattr__(ode.name, ode)
             self.ODEs.append(ode)
@@ -1435,16 +1417,3 @@ class NewCore(NewCoreBase):
 
     def hasCompartments(self):
         return MapList(c.name for c in self.compartments)
-
-##  if __psyco_active__:
-    ##  psyco.bind(NewCoreBase)
-    ##  psyco.bind(NumberBase)
-    ##  psyco.bind(Species)
-    ##  psyco.bind(Parameter)
-    ##  psyco.bind(AssignmentRule)
-    ##  psyco.bind(Reaction)
-    ##  psyco.bind(ODE)
-    ##  psyco.bind(NewCore)
-
-
-
