@@ -7,48 +7,47 @@ This module provides functions for Stochastic Simulation Algorithms Analysis (SS
 
 Written by TR Maarleveld, Amsterdam, The Netherlands
 E-mail: tmd200@users.sourceforge.net
-Last Change: May 17, 2013
-"""
+Last Change: October 31, 2014
+"""  
+from __future__ import division, print_function, absolute_import
 
-try:    
-    import matplotlib.pyplot as plt  
-    import matplotlib.colors as clr
-    IS_PLOTTING = True
-except: 
-    IS_PLOTTING = False
-    print "Matplotlib module is not available, thus it is impossible to create plots"
+from stochpy import _IsPlotting
+if _IsPlotting:
+    from stochpy import plt  
+    from stochpy import matplotlib
+    from matplotlib import gridspec,colors as clr
 
-try: 
+from stochpy import _IsNumPy
+if _IsNumPy:
     import numpy as np
-except:
-    print "Error: NumPy module is not available. "
+else:
     sys.exit()
 
 import copy,sys
 
-def getDataForTimeSimPlot(Arr_data,n_points2plot = 10000):
+def getDataForTimeSimPlot(Arr_data,n_points2plot = 100000):
     """
-    getDataForTimeSimPlot(data,n_points2plot = 10000)
+    getDataForTimeSimPlot(data,n_points2plot = 100000)
     
     Input:
      - *Arr_data* (numpy array)
      - *n_points2plot* [default = 10000] (integer)
     """    
     len_data = len(Arr_data)    
-    if (len_data > 100000): # use n_points2plot only if datasets become too large     
+    if (len_data > n_points2plot): # use n_points2plot only if datasets become too large    
         L_data2plot = [Arr_data[0]]
-        step_size = len_data/int(abs(n_points2plot))        
-        for i in xrange(step_size,len_data,step_size):
-            t = Arr_data[i][0]            
-            data_point = copy.deepcopy(list(L_data2plot[-1][1:]))
+        step_size = len_data//int(abs(n_points2plot))        
+        for i in range(step_size,len_data,step_size):
+            t = Arr_data[i][0]        
+            data_point = copy.deepcopy(L_data2plot[-1][1:].tolist())
             data_point.insert(0,t)
             L_data2plot.append(data_point)
             L_data2plot.append(Arr_data[i])
-        print "Info: Plotting %s out of %s points.\nUse the argument 'n_points2plot' to alter the number of plotted events.\nUse n_points2plot='All' to plot all the data points."  % (n_points2plot,len_data)
+        print("Info: Plotting {0:d} out of {1:d} points. Use the argument 'n_points2plot' to alter the number of plotted events.".format(n_points2plot,len_data) )        
     else: 
-        L_data2plot = copy.deepcopy(list(Arr_data))
+        L_data2plot = copy.deepcopy(Arr_data.tolist())
         j=1       
-        for i in xrange(1,len_data):
+        for i in range(1,len_data):
             t = Arr_data[i][0]
             data_prev = copy.deepcopy(Arr_data[i-1]) # data of previous ...
             data_prev[0] = t
@@ -67,78 +66,90 @@ def Count(L_data,L_edges):
     n_edges = len(L_edges)
     L_output = np.zeros(n_edges)
     for value in L_data:    
-        for i in xrange(n_edges-1):
+        for i in range(n_edges-1):
             if (value >= L_edges[i]) and (value < L_edges[i+1]):
                 L_output[i]+=1  
     return np.array(L_output)  
     
-def GetDataDistributions(L_sim_output,L_names):  
+def GetDataDistributions(sim_output,species):  
     """
     Get distributions, means, standard deviations, and the (raw) moments
       
     Input:
-    - *L_sim_output* (list)
-    - *L_names* (list)
-    """     
-    n_names = len(L_names)
-    L_distributions = [{} for i in xrange(n_names)]
-    endtime = L_sim_output[-1][0]
-    for timestep in xrange(len(L_sim_output)-1):
-        for i in xrange(n_names): 
-            try:
-                L_distributions[i][L_sim_output[timestep][i+1]] += L_sim_output[timestep+1][0] - L_sim_output[timestep][0]
-            except KeyError:
-                L_distributions[i][L_sim_output[timestep][i+1]] = L_sim_output[timestep+1][0] - L_sim_output[timestep][0]  
+    - *sim_output* (list)
+    - *species* (list)    
     
-    L_probability_mass = []
+    Mean = mu = sum(x*P(x))
+    Variance = sum(x^2 * p(x)) - mu**2
+    
+    Output:
+    - *L_probability_mass*
+    - *D_means*
+    - *D_stds*
+    - *D_moments*
+    """
+    n_species = len(species)
+    L_distributions = [{} for i in range(n_species)]
+    starttime = sim_output[0][0]
+    endtime = sim_output[-1][0]
+    n_datapoints = len(sim_output)
     D_means = {}
     D_stds = {}
     D_moments = {}
-    for i in xrange(n_names):
-        L_x_i = np.array(sorted(L_distributions[i]))     
-        L_y_i = np.array([L_distributions[i][x_i] for x_i in sorted(L_distributions[i])])/float(endtime) # probability = dt/T
-        mean = (L_x_i*L_y_i).sum()
-        mean_sq = (L_x_i**2*L_y_i).sum()
-        var = mean_sq - mean*mean
-        std = var**0.5
-        L_probability_mass.append([L_x_i,L_y_i])
-        id_name = L_names[i]
-         
-        D_means[id_name] = mean
-        D_stds[id_name] = std
-          
-        D_moments[id_name] = {}
-        D_moments[id_name]['1'] = mean 
-        D_moments[id_name]['2'] = mean_sq
-        D_moments[id_name]['3'] = (L_x_i**3*L_y_i).sum()
-        D_moments[id_name]['4'] = (L_x_i**4*L_y_i).sum()        
-    del L_sim_output,L_distributions
+    L_probability_mass = []
+    if n_datapoints > 1:
+        for t in range(n_datapoints-1):
+            for i in range(n_species):                 
+                try:
+                    L_distributions[i][int(sim_output[t][i+1])] += sim_output[t+1][0] - sim_output[t][0]
+                except KeyError:
+                    L_distributions[i][int(sim_output[t][i+1])] = sim_output[t+1][0] - sim_output[t][0]         
+        for i in range(n_species):
+            s_id = species[i]
+            x = np.array(sorted(L_distributions[i]),dtype=int)            
+            p_x = np.array([L_distributions[i][x_i] for x_i in x])/float(endtime-starttime) # probability = dt/T   
+            
+            mu = (x*p_x).sum()           
+            mu_sq = (x**2*p_x).sum()
+            var = mu_sq - mu**2
+            std = var**0.5
+            L_probability_mass.append([x,p_x])            
+             
+            D_means[s_id] = mu
+            D_stds[s_id] = std
+              
+            D_moments[s_id] = {}
+            D_moments[s_id]['1'] = mu
+            D_moments[s_id]['2'] = mu_sq
+            D_moments[s_id]['3'] = (x**3*p_x).sum()
+            D_moments[s_id]['4'] = (x**4*p_x).sum()      
+        
     return (L_probability_mass,D_means,D_stds,D_moments)        
     
 def Binning(x,y,bin_size):
     """
     Binning(x,y,bin_size)
     
-    Binning of the PDF
+    Binning of the PDF. Works only properly if there is a fixed distance 
     
     bin_size is the number of indices for binning (23/01/2013)   
     
     Input:
-     - *x* list of x-values
-     - *y* list of probabilities for each x[i]
+     - *x* (list) of x-values
+     - *y* (list) of probabilities for each x[i]
      - *bin_size* (integer)     
     """   
     L_binned_data = []
     nprobabilities = len(x)
-    if type(bin_size) == int:
+    if isinstance(bin_size,int):
          pass
-    elif type(bin_size) == float:
-         print "Warning: The 'bin_size' must an integer rather than a float; float %s is rounded to %s" % (bin_size,int(bin_size))
+    elif isinstance(bin_size,float):
+         print("*** WARNING ***: The 'bin_size' must an integer rather than a float; float {0} is rounded to {0:d}".format(bin_size,int(bin_size)))
          bin_size = int(bin_size)
     else:
-        print "Error: The 'bin_size' must be an integer"
-        sys.exit()
-    if (nprobabilities > 1) and (type(bin_size) == int):        # binning is possible    
+        raise Warning("The 'bin_size' must be an integer")
+        
+    if (nprobabilities > 1) and isinstance(bin_size,int):        # binning is possible    
         xdif = x[1]-x[0]
         if (bin_size <= 1) or (bin_size > nprobabilities):      # no binning because of the bin size
             data = np.array([x,y]).transpose()            
@@ -146,7 +157,7 @@ def Binning(x,y,bin_size):
         elif (bin_size > 1):                                    # do binning            
             max_index = y.index(max(y))                         # maximum prob index
             ### Get interval around the maximum value ###
-            L_interval = range(int(max_index-(bin_size/2.0)+1),max_index+int(round(bin_size/2.0)+1)) # Get interval around maximum prob
+            L_interval = list(range(int(max_index-(bin_size/2.0)+1),max_index+int(round(bin_size/2.0)+1))) # Get interval around maximum prob
             if (bin_size%2):
                 L_interval.pop()
             index_lb = L_interval[0]            
@@ -161,7 +172,7 @@ def Binning(x,y,bin_size):
             L_binned_data.append([x[L_interval[-1]]+xdif/2.0 ,sum(y[L_interval[0]:L_interval[-1]+1])/float(bin_size)])           
             L_individuals = []                                  # locations with bin_size = 1
             if L_interval[0]:
-                nbins1 = (L_interval[0]-1)/bin_size             # nbins before L_interval around max prob
+                nbins1 = (L_interval[0]-1)//bin_size             # nbins before L_interval around max prob
                 start1 = L_interval[0]-1 - nbins1*bin_size      # start location of binning before interval
                 if L_interval[0] == 1:
                     L_individuals.append(0)
@@ -171,15 +182,15 @@ def Binning(x,y,bin_size):
                 start1 = 0
                 L_individuals.append(0)
             start2 = L_interval[-1]                             # start location of binning after interval
-            nbins2 = (nprobabilities-1 - start2)/bin_size       # nbins after interval around max prob
-            L_individuals += [i for i in xrange(0,start1)]        
-            L_individuals += [i for i in xrange(start2+1+(bin_size*nbins2),nprobabilities)]
+            nbins2 = (nprobabilities-1 - start2)//bin_size       # nbins after interval around max prob            
+            L_individuals += [i for i in range(0,start1)]        
+            L_individuals += [i for i in range(start2+1+(bin_size*nbins2),nprobabilities)]
             x_ = np.array(x)
             y_ = np.array(y)
-            for i in xrange(nbins1):            
+            for i in range(nbins1):            
                 L_binned_data.append([x_[start1+bin_size]+xdif/2.0 ,sum(y_[start1:start1+bin_size])/float(bin_size)])
                 start1+=bin_size
-            for i in xrange(nbins2):  
+            for i in range(nbins2):  
                 L_binned_data.append([x[start2+bin_size]+xdif/2.0 ,sum(y[start2:start2+bin_size])/float(bin_size)]) 
                 start2+=bin_size                  
             for i in L_individuals:
@@ -205,21 +216,25 @@ def LogBin(L_data,factor):
     """
     xmin = float(min(L_data))    
     nbins = int(np.ceil(np.log(max(L_data)/xmin)/np.log(factor)))  
-    L_edges = np.zeros(nbins)
-    L_edges[0] = xmin     
-    for i in xrange(1,nbins):
-        L_edges[i] = L_edges[i-1]*factor  
-    L_x  = L_edges[0:(nbins-1)]+np.diff(L_edges)/2  
-    L_dp = Count(L_data,L_edges)
-    L_ry = np.array(L_dp[0:(nbins-1)])  
-    L_dedges = np.array(np.diff(L_edges))  
-    L_y = L_ry/(sum(L_ry)*L_dedges) 
+    L_x = None
+    L_y = None    
+    if nbins:
+        L_edges = np.zeros(nbins)
+        L_edges[0] = xmin     
+        for i in range(1,nbins): # 1,nbins
+            L_edges[i] = L_edges[i-1]*factor  
+        
+        L_x  = L_edges[0:(nbins-1)]+np.diff(L_edges)/2  
+        L_dp = Count(L_data,L_edges)
+        L_ry = np.array(L_dp[0:(nbins-1)])  
+        L_dedges = np.array(np.diff(L_edges))  
+        L_y = L_ry/(sum(L_ry)*L_dedges)        
     return(L_x,L_y,nbins)
 
 def ObtainWaitingtimes(data_stochsim,L_reactions): 
     """    
     ObtainWaitingtimes(data_stochsim,L_reactions)
-    
+
     This function extracts the waiting times for each reaction of the model from the used SSA output.
 
     Input:
@@ -227,7 +242,7 @@ def ObtainWaitingtimes(data_stochsim,L_reactions):
      - *L_reactions* (list)
     output:
      - *D_waiting_times* (dict) 
-  
+    
     Note: It is impossible to use this function in combination with the Tau-leaping method, because the Tau-Leaping results are not exact!
     """
     L_time = data_stochsim.time.flatten()
@@ -238,32 +253,41 @@ def ObtainWaitingtimes(data_stochsim,L_reactions):
     for r_id in L_reactions:     
         D_waiting_times[r_id] = []                                 # create a list that will contain event waiting times for reaction r
         
-    ### np.nan (e.g. events) is ignored, because we determine it only for fired reactions like (1,2,3, .... nreactions)
-    for (current_time,r_index) in zip(L_time[1:],L_fired_reactions): # fixed January 21 2013        
-        for i in xrange(1,nreactions+1):                           # fired reactions are (1,2,3, .... nreactions)
+    for (current_time,r_index) in zip(L_time[1:],L_fired_reactions[1:]): # Updated Oktober 1st
+        for i in range(1,nreactions+1):                            # fired reactions are (1,2,3, .... nreactions)
             if r_index == i:
-                if r_index in D_last_time_fired:
-                    D_last_time_fired[r_index]                     # A previous firing time is necessary                                  
-                    D_waiting_times[L_reactions[int(r_index-1)]].append(current_time-D_last_time_fired[r_index]) # Add inter-arrival time
+                if r_index in D_last_time_fired:  
+                    r_name = L_reactions[int(r_index-1)]               
+                    D_waiting_times[r_name].append(current_time - D_last_time_fired[r_index]) # Add inter-arrival time
                     D_last_time_fired[r_index] = current_time      # Update last firing time     
                 else:
                     D_last_time_fired[r_index] = current_time      # Initial firing time
+
+            elif r_index == -i:                                    # Handle delayed completions 01-10-2014
+                r_name_compl = L_reactions[ int(abs(r_index)-1) ] + '_Completion'         
+                if r_index in D_last_time_fired:                                  
+                    D_waiting_times[r_name_compl].append(current_time - D_last_time_fired[r_index]) # Add inter-arrival time
+                    D_last_time_fired[r_index] = current_time      # Update last firing time     
+                else:
+                    D_last_time_fired[r_index] = current_time      # Initial firing time
+                    D_waiting_times.setdefault(r_name_compl, [])   # Set keyname if not present\
+        #print current_time,D_last_time_fired
     return D_waiting_times
 
-def GetAverageResults(L_regular_grid):
+def GetAverageResults(regular_grid):
     """
-    GetAverageResults(L_regular_grid)
+    GetAverageResults(regular_grid)
     
     Gets the averaged output of multiple trajectories
 
     Input: 
-     - *L_regular_grid* (nested list)
+     - *regular_grid* (nested list)
     """
     L_means = []
     L_stds = []
-    for L_data in L_regular_grid:
-        L_means.append(np.mean(L_data,0))
-        L_stds.append(np.std(L_data,0))    
+    for data in regular_grid:
+        L_means.append(np.mean(data,0))
+        L_stds.append(np.std(data,0))    
     return (L_means,L_stds)
     
 def RemoveBias(x, axis):
@@ -401,10 +425,9 @@ def Autocorrelation(s, **kwargs):
     else:
         sxx_0 = sxx[0] 
     if not sxx_0:
-        sxx = [np.nan for i in xrange(len(sxx))] # Modification
+        sxx = [np.nan for i in range(len(sxx))] # Modification
     else:    
-        sxx /= sxx_0
-    #print sxx_0    
+        sxx /= sxx_0    
     return sxx
 
 class DoPlotting():
@@ -428,7 +451,7 @@ class DoPlotting():
       """ Reset figure numbers if trajectories > 1 """
       self.plotnum = 1 
       
-  def TimeSeries(self,data,n_points2plot,datatype2plot,L_labels,trajectory_index,linestyle,marker,colors,title,xlabel,ylabel,IsLegend):      
+  def TimeSeries(self,data,n_points2plot,datatype2plot,L_labels,trajectory_index,linestyle,linewidth,marker,colors,title,xlabel,ylabel,IsLegend,legend_location):      
       """
       TimeSeries(data,n_points2plot,datatype2plot,trajectory_index,linestyle,marker,colors,title)
       
@@ -441,13 +464,15 @@ class DoPlotting():
        - *L_labels* (list)
        - *trajectory_index* (integer)
        - *linestyle* (string)
+       - *linewidth* (float)
        - *title* (string)
        - *xlabel* (string)
        - *ylabel* (string)
-       - *IsLegend* (boolean)
+       - *IsLegend* (boolean)       
       """
-      plt.figure(self.plotnum)      
-      datatype2plot_indices = [L_labels.index(datatype) for datatype in datatype2plot]      
+      plt.figure(self.plotnum)       
+      datatype2plot_indices = [L_labels.index(datatype) for datatype in datatype2plot]
+            
       L_data = getDataForTimeSimPlot(data,n_points2plot)
       L_time = L_data[:,0]   
       if len(datatype2plot) == 1:
@@ -465,31 +490,33 @@ class DoPlotting():
                   j=0
 
           if colors == None:
-              plt.plot(L_time,y,marker,ls = linestyle,color = self.colors[j])
+              plt.plot(L_time,y,marker,ls = linestyle,lw = linewidth,color = self.colors[j])
           else:
               if clr.is_color_like(colors[j]):
-                  plt.plot(L_time,y,marker,ls = linestyle,color = colors[j])
+                  plt.plot(L_time,y,marker,ls = linestyle,lw = linewidth,color = colors[j])
               else:
-                  print "Warning: '%s' is not recognized as a valid color code" % colors[j]
-                  plt.plot(L_time,y,marker,ls = linestyle,color = self.colors[j])
+                  print("*** WARNING ***: '{0}' is not recognized as a valid color code".format(colors[j]) )
+                  plt.plot(L_time,y,marker,ls = linestyle,lw = linewidth,color = self.colors[j])
                   colors = None
           j+=1
       if IsLegend:
-          plt.legend(datatype2plot,numpoints=1,frameon=True)
+          plt.legend(datatype2plot,numpoints=1,frameon=True,loc=legend_location)
       plt.title(title)
       plt.xlabel(xlabel) 
       plt.ylabel(ylabel)      
       
-  def Autocorrelations(self,lags,data,datatype2plot,L_labels,trajectory_index,linestyle,marker,colors,title,xlabel,ylabel,IsLegend):      
+  def Autocorrelations(self,lags,data,datatype2plot,L_labels,trajectory_index,linestyle,linewidth,marker,colors,title,xlabel,ylabel,IsLegend,legend_location):      
       """
-      Autocorrelations(data,species2plot,trajectory_index,linestyle,marker,colors,title)
+      Autocorrelations(lags,data,species2plot,trajectory_index,linestyle,marker,colors,title,xlabel,ylabel,IsLegend,legend_location)
 
       Input:
+       - *lags*
        - *data* (array)      
        - *datatype2plot* (list) 
        - *L_labels* (list)
        - *trajectory_index* (integer)       
        - *linestyle* (string)
+       - *linewidth* (float)
        - *marker* string)
        - *colors* (list)
        - *title* (string)
@@ -514,24 +541,24 @@ class DoPlotting():
                                   
           y = data[i][0:len(lags)]          
           if colors == None:
-              plt.plot(lags,y,marker,ls = linestyle,color = self.colors[j])
+              plt.plot(lags,y,marker,ls = linestyle,lw = linewidth, color = self.colors[j])
           else:   
               if clr.is_color_like(colors[j]):             
-                  plt.plot(lags,y,marker,ls = linestyle,color = colors[j])
+                  plt.plot(lags,y,marker,ls = linestyle,lw = linewidth, color = colors[j])
               else:
-                  print "Warning: '%s' is not recognized as a valid color code" % colors[j]
-                  plt.plot(lags,y,marker,ls = linestyle,color = self.colors[j]) 
+                  print("*** WARNING ***: '{0}' is not recognized as a valid color code".format(colors[j]) )
+                  plt.plot(lags,y,marker,ls = linestyle,lw = linewidth, color = self.colors[j]) 
                   colors = None
           j+=1 
       if IsLegend:    
-          plt.legend(datatype2plot,numpoints=1,frameon=True)      
+          plt.legend(datatype2plot,numpoints=1,frameon=True,loc=legend_location)      
       plt.title(title)
       plt.xlabel(xlabel) 
       plt.ylabel(ylabel)
       
-  def Distributions(self,distributions,datatype2plot,L_labels,trajectory_index,linestyle,linewidth,colors,title,xlabel,ylabel,IsLegend,bin_size):    
+  def Distributions(self,distributions,datatype2plot,L_labels,trajectory_index,linestyle,linewidth,colors,title,xlabel,ylabel,IsLegend,legend_location,bin_size):    
       """
-      Distributions(distributions,datatype2plot,trajectory_index,linestyle,colors,title,bin_size)
+      Distributions(distributions,datatype2plot,trajectory_index,linestyle,colors,title,xlabel,ylabel,IsLegend,legend_location,legend_location,bin_size)
       
       Plots the distributions of species and/or propensities
 
@@ -540,11 +567,14 @@ class DoPlotting():
        - *datatype2plot* (list)
        - *L_labels* (list)
        - *trajectory_index* (integer)
+       - *linestyle* (string)
+       - *linewidth* (float)            
        - *colors* (list)
        - *title* (string)
        - *xlabel* (string)
        - *ylabel* (string)       
        - *IsLegend* (boolean)
+       - *legend_location* [default = 'upper right'] (string/integer)
        - *bin_size* (string)
       """ 
       plt.figure(self.plotnum)
@@ -555,8 +585,8 @@ class DoPlotting():
           j=0
 
       for i in datatype2plot_indices: 
-          x = list(copy.copy(distributions[i][0]))
-          y = list(copy.copy(distributions[i][1]))          
+          x = copy.copy(distributions[i][0].tolist())
+          y = copy.copy(distributions[i][1].tolist())          
           L_data = Binning(x,y,bin_size)
           if colors == None:              
               if j >= len(self.colors):                  
@@ -570,19 +600,19 @@ class DoPlotting():
              if clr.is_color_like(colors[j]):             
                   plt.step(L_data[:,0],L_data[:,1],ls = linestyle,lw = linewidth,color = colors[j])     
              else:
-                  print "Warning: '%s' is not recognized as a valid color code" % colors[j]
+                  print("*** WARNING ***: '{0}' is not recognized as a valid color code".format(colors[j]) )
                   plt.step(L_data[:,0],L_data[:,1],ls = linestyle,lw = linewidth,color = self.colors[j])
                   colors = None
           j+=1
       if IsLegend:
-          plt.legend(datatype2plot,numpoints=1,frameon=True)
+          plt.legend(datatype2plot,numpoints=1,frameon=True,loc=legend_location)
       plt.title(title)
       plt.xlabel(xlabel)
       plt.ylabel(ylabel)            
 
-  def WaitingtimesDistributions(self,D_waiting_times,rates2plot,trajectory_index,linestyle,marker,colors,title,xlabel,ylabel,IsLegend):
+  def WaitingtimesDistributions(self,D_waiting_times,rates2plot,trajectory_index,linestyle,linewidth, marker,colors,title,xlabel,ylabel,IsLegend,legend_location):
       """
-      WaitingtimesDistributions(D_waiting_times,rates2plot,trajectory_index,linestyle,marker,colors,title)
+      WaitingtimesDistributions(D_waiting_times,rates2plot,trajectory_index,linestyle,linewidth, marker,colors,title,xlabel,ylabel,IsLegend,legend_location)
       
       Plots the waiting times for each reaction in the model. Makes use of ObtainWaitingtimes to derive the waiting times out of the SSA output.
  
@@ -591,10 +621,14 @@ class DoPlotting():
        - *rates2plot* (list)
        - *trajectory_index* (integer)
        - *linestyle* (string)
+       - *linewith* (float)    
+       - *marker* (string)
+       - *colors* (list)
        - *title* (string)
        - *xlabel* (string)
        - *ylabel* (string)
        - *IsLegend* (boolean)
+       - *legend_location* [default = 'upper right'] (string/integer)
       """
       plt.figure(self.plotnum)               
       if len(rates2plot) == 1:
@@ -605,34 +639,36 @@ class DoPlotting():
       L_legend_names = []
       for r_id in rates2plot:                        
           L_waiting_times = D_waiting_times[r_id]       # get list of waiting times for a given reaction
-          if len(L_waiting_times) > 1:			# At least 2 waiting times are necessary per reaction
+          if len(L_waiting_times) > 1:			            # At least 2 waiting times are necessary per reaction
               (x,y,nbins) = LogBin(L_waiting_times,1.5) # Create logarithmic bins (HARDCODED 1.5)
-              if colors == None:              
-                  if j >= len(self.colors):                  
-                      j=0
-              elif colors:
-                  if j >= len(colors):
-                      j=0                 
-              if colors == None:
-                  plt.loglog(x,y,marker,ls = linestyle,color = self.colors[j])
-              else:
-                  if clr.is_color_like(colors[j]):           
-                      plt.loglog(x,y,marker,ls = linestyle,color = colors[j])
+              
+              if x != None:
+                  if colors == None:              
+                      if j >= len(self.colors):                  
+                          j=0
+                  elif colors:
+                      if j >= len(colors):
+                          j=0                 
+                  if colors == None:
+                      plt.loglog(x,y,marker,ls = linestyle,lw=linewidth,color = self.colors[j])
                   else:
-                      print "Warning: '%s' is not recognized as a valid color code" % colors[j]
-                      plt.loglog(x,y,marker,ls = linestyle,color = self.colors[j])
-                      colors = None                            
-              L_legend_names.append(r_id)
-              j+=1
+                      if clr.is_color_like(colors[j]):           
+                          plt.loglog(x,y,marker,ls = linestyle,lw=linewidth,color = colors[j])
+                      else:
+                          print("*** WARNING ***: '{0}' is not recognized as a valid color code".format(colors[j]) )
+                          plt.loglog(x,y,marker,ls = linestyle,lw=linewidth,color = self.colors[j])
+                          colors = None                            
+                  L_legend_names.append(r_id)
+                  j+=1
       plt.title(title)
       plt.xlabel(xlabel)
       plt.ylabel(ylabel)
       if IsLegend:
-          plt.legend(L_legend_names,numpoints=1,frameon=True)
+          plt.legend(L_legend_names,numpoints=1,frameon=True,loc=legend_location)
       
-  def AverageTimeSeries(self,L_means,L_stds,L_time,nstd,datatype2plot,L_labels,linestyle,marker_,colors,title,xlabel,ylabel,IsLegend):
+  def AverageTimeSeries(self,L_means,L_stds,L_time,nstd,datatype2plot,L_labels,linestyle,linewidth,marker_,colors,title,xlabel,ylabel,IsLegend,legend_location):
       """
-      AverageSpeciesTimeSeries(L_means,L_stds,L_time,datatype2plot,linestyle,marker_,colors,title)
+      AverageSpeciesTimeSeries(L_means,L_stds,L_time,nstd,datatype2plot,L_labels,linestyle,linewidth,marker_,colors,title,xlabel,ylabel,IsLegend,legend_location)
       
       Plots the average and standard deviation of datatype on a regular grid
 
@@ -645,10 +681,14 @@ class DoPlotting():
        - *L_labels* (list)
        - *L_time* (list)
        - *linestyle* (string)
+       - *linewidth* (float)
+       - *marker_* (string)
+       - *colors* (list)       
        - *title* (string)
        - *xlabel* (string)
        - *ylabel* (string)
        - *IsLegend* (boolean)
+       - *legend_location* [default = 'upper right'] (string/integer)
       """ 
       assert nstd > 0, "Error: The number of STDs must be a value larger than zero"
       plt.figure(self.plotnum)
@@ -663,22 +703,22 @@ class DoPlotting():
                   j=0
           npoints = len(L_time)
           if colors == None:
-              plt.errorbar(L_time,L_means[i][0:npoints],yerr = nstd*np.array(L_stds[i][0:npoints]),color = self.colors[j],ls = linestyle,marker = marker_,label = L_labels[i]) # plot with y-axis error bars
+              plt.errorbar(L_time,L_means[i][0:npoints],yerr = nstd*np.array(L_stds[i][0:npoints]),color = self.colors[j],ls = linestyle,lw=linewidth,marker = marker_,label = L_labels[i]) # plot with y-axis error bars
           else:
               if clr.is_color_like(colors[j]):     
-                  plt.errorbar(L_time,L_means[i][0:npoints],yerr = nstd*np.array(L_stds[i][0:npoints]),color = colors[j],ls = linestyle,marker = marker_,label = L_labels[i])
+                  plt.errorbar(L_time,L_means[i][0:npoints],yerr = nstd*np.array(L_stds[i][0:npoints]),color = colors[j],ls = linestyle,lw=linewidth,marker = marker_,label = L_labels[i])
               else:
-                  print "Warning: '%s' is not recognized as a valid color code" % colors[j]
-                  plt.errorbar(L_time,L_means[i][0:npoints],yerr = nstd*np.array(L_stds[i][0:npoints]),color = self.colors[j],ls = linestyle,marker = marker_,label = L_labels[i])
+                  print("*** WARNING ***: '{0}' is not recognized as a valid color code".format(colors[j]) )
+                  plt.errorbar(L_time,L_means[i][0:npoints],yerr = nstd*np.array(L_stds[i][0:npoints]),color = self.colors[j],ls = linestyle,lw=linewidth,marker = marker_,label = L_labels[i])
                   colors = None
           j+=1
       if IsLegend:
-          plt.legend(numpoints=1,frameon=True)
+          plt.legend(numpoints=1,frameon=True,loc=legend_location)
       plt.title(title)
       plt.xlabel(xlabel)
       plt.ylabel(ylabel)
 
-  def AverageDistributions(self,L_means,L_stds,nstd,datatype2plot,L_labels,linestyle,marker_,colors,title,xlabel,ylabel,IsLegend):
+  def AverageDistributions(self,L_means,L_stds,nstd,datatype2plot,L_labels,linestyle,linewidth,marker_,colors,title,xlabel,ylabel,IsLegend,legend_location):
       """      
       Plots the average and standard deviation
 
@@ -688,10 +728,14 @@ class DoPlotting():
        - *nstd* (float)
        - *L_labels* (list)
        - *linestyle* (string)
+       - *linewidth* (float)
+       - *marker_* (string)
+       - *colors* (list)
        - *title* (string)
        - *xlabel* (string)
        - *ylabel* (string)
        - *IsLegend* (boolean)
+       - *legend_location* [default = 'upper right'] (string/integer)
       """
       assert nstd > 0, "Error: The number of STDs must be a value larger than zero"     
       plt.figure(self.plotnum)
@@ -705,22 +749,22 @@ class DoPlotting():
               if j >= len(colors):
                   j=0 
           if colors == None:              
-              plt.errorbar(L_means[i][0],L_means[i][1],yerr = nstd * np.array(L_stds[i][1]),color = self.colors[j],ls = linestyle,marker = marker_,label = L_labels[i]) # plot with y-axis error bars
+              plt.errorbar(L_means[i][0],L_means[i][1],yerr = nstd * np.array(L_stds[i][1]),color = self.colors[j],ls = linestyle,lw = linewidth,marker = marker_,label = L_labels[i]) # plot with y-axis error bars
           else:
               if clr.is_color_like(colors[j]):     
-                  plt.errorbar(L_means[i][0],L_means[i][1],yerr = nstd*np.array(L_stds[i][1]),color = colors[j],ls = linestyle,marker = marker_,label = L_labels[i])
+                  plt.errorbar(L_means[i][0],L_means[i][1],yerr = nstd*np.array(L_stds[i][1]),color = colors[j],ls = linestyle,lw = linewidth,marker = marker_,label = L_labels[i])
               else:
-                  print "Warning: '%s' is not recognized as a valid color code" % colors[j]
-                  plt.errorbar(L_means[i][0],L_means[i][1],yerr = nstd * np.array(L_stds[i][1]),color = self.colors[j],ls = linestyle,marker = marker_,label = L_labels[i])
+                  print("*** WARNING ***: '{0}' is not recognized as a valid color code".format(colors[j]) )
+                  plt.errorbar(L_means[i][0],L_means[i][1],yerr = nstd * np.array(L_stds[i][1]),color = self.colors[j],ls = linestyle,lw = linewidth,marker = marker_,label = L_labels[i])
                   colors = None                  
           j+=1
       if IsLegend:
-          plt.legend(numpoints=1,frameon=True)
+          plt.legend(numpoints=1,frameon=True,loc=legend_location)
       plt.title(title)
       plt.xlabel(xlabel)
       plt.ylabel(ylabel)
       
-  def AverageDistributionsCI(self,L_means,L_stds,nstd,datatype2plot,L_labels,colors,title,xlabel,ylabel,IsLegend):
+  def AverageDistributionsCI(self,L_means,L_stds,nstd,datatype2plot,L_labels,colors,title,xlabel,ylabel,IsLegend,legend_location):
       assert nstd > 0, "Error: The number of STDs must be a value larger than zero"      
       plt.figure(self.plotnum)
       datatype2plot_indices = [L_labels.index(datatype) for datatype in datatype2plot]           
@@ -737,7 +781,7 @@ class DoPlotting():
           X_i = []
           Y_i = []
           L_errors = []
-          for j in xrange(len(L_s_amount)):
+          for j in range(len(L_s_amount)):
               if (not L_s_amount[j] == L_s_amount[0]) and (not L_s_amount[j] == L_s_amount[-1]):
                   X_i.append(L_s_amount[j])
                   Y_i.append(L_mu[j-1])
@@ -755,19 +799,19 @@ class DoPlotting():
               if j >= len(colors):
                   j=0 
           if colors == None:                         
-              plt.fill(X_e-0.5,Y_e,  alpha=.25, ec='None', label='%s STD confidence interval' % nstd,color = self.colors[j]) 
+              plt.fill(X_e-0.5,Y_e,  alpha=.25, ec='None', label='{0:d} STD confidence interval'.format(nstd),color = self.colors[j]) 
               plt.plot(np.array(X_i)-0.5,np.array(Y_i),color = self.colors[j])
           else:
               if clr.is_color_like(colors[j]):     
-                  plt.fill(X_e-0.5,Y_e,  alpha=.25, ec='None', label='%s STD confidence interval' % nstd,color = colors[j]) 
+                  plt.fill(X_e-0.5,Y_e,  alpha=.25, ec='None', label='{0:d} STD confidence interval'.format(nstd),color = colors[j]) 
                   plt.plot(np.array(X_i)-0.5,np.array(Y_i),color = colors[j])
               else:
-                  print "Warning: '%s' is not recognized as a valid color code" % colors[j]
-                  plt.fill(X_e-0.5,Y_e,  alpha=.25, ec='None', label='%s STD confidence interval' % nstd,color = self.colors[j]) 
+                  print("*** WARNING ***: '{0}' is not recognized as a valid color code".format(colors[j]) )
+                  plt.fill(X_e-0.5,Y_e,  alpha=.25, ec='None', label='{0:d} STD confidence interval'.format(nstd),color = self.colors[j]) 
                   plt.plot(np.array(X_i)-0.5,np.array(Y_i),color = self.colors[j])      
                   colors = None
       if IsLegend:
-          plt.legend(numpoints=1,frameon=True)
+          plt.legend(numpoints=1,frameon=True,loc=legend_location)
       plt.xlabel(xlabel)
       plt.ylabel(ylabel)
       plt.title(title)
