@@ -7,9 +7,10 @@ Module that contains functions that are created by the users of stochpy. New fun
 
 Written by T.R. Maarleveld, Amsterdam, The Netherlands
 E-mail: tmd200@users.sourceforge.net
-Last Change: May 21, 2013
+Last Change: October 31, 2014
 """
 
+from __future__ import division, print_function, absolute_import
 import stochpy as _stochpy_
 import copy,numpy as np
 mod = None
@@ -47,9 +48,9 @@ def GetAnalyticalWaitingtimes(kon,koff,ksyn):
 def doSequentialSim(smod,n_generations,cell_division_times):
     """ Model protein synthesis subject to cell division """
     ### Start sequential modelling part ###
-    for i in xrange(1,n_generations):     
+    for i in range(1,n_generations):     
         ### divide each species between two daughter cells ###
-        for j in xrange(0,len(smod.data_stochsim.species_labels)): 
+        for j in range(0,len(smod.data_stochsim.species_labels)): 
             species_amount = smod.SSA.sim_output[-1][1:][j]                 
             if species_amount:
                 smod.settings.X_matrix[j] = np.random.binomial(n=species_amount,p=0.5,size=1)
@@ -70,12 +71,12 @@ def doSequentialSim(smod,n_generations,cell_division_times):
 
 class Utils():
     def __init__(self):
-        print "See http://stochpy.sf.net/examples.html for more explanation about these examples"
+        print("See http://stochpy.sf.net/examples.html for more explanation about these examples")
         global mod
         mod = _stochpy_.SSA()        
 
-    def doExample1(self):
-        """ Immigration Death example (available at http://stompy.sourceforge.net/examples.html) """        
+    def DoExample1(self):
+        """ Immigration Death example (available at http://stochpy.sourceforge.net/examples.html) """        
         mod.Model('ImmigrationDeath.psc')  # Ksyn = 10, Kdeg = 0.2, and mRNA(init) = 50
         lambda_ = 50
         N = 1000000
@@ -86,15 +87,15 @@ class Utils():
         mod.PrintSpeciesMeans()
         mod.PrintSpeciesStandardDeviations()        
     
-    def doExample2(self):
-        """ SBML events and Interpolation example (available at http://stompy.sourceforge.net/examples.html) """
+    def DoExample2(self):
+        """ SBML events and Interpolation example (available at http://stochpy.sourceforge.net/examples.html) """
         mod.Model('dsmts-003-04.xml.psc')
         mod.DoStochSim(end = 50,mode = 'time',trajectories = 1000)
         mod.PlotAverageSpeciesTimeSeries()
         mod.PrintAverageSpeciesTimeSeries()
         
-    def doExample3(self):
-        """ Burstmodel example (available at http://stompy.sourceforge.net/examples.html) """
+    def DoExample3(self):
+        """ Burstmodel example (available at http://stochpy.sourceforge.net/examples.html) """
         mod.Model('Burstmodel.psc')  # Parameter values in Burstmodel.psc: kon = koff = 0.05
         ntimesteps = 1500000        
         mod.ChangeParameter("kon",0.05)
@@ -146,8 +147,8 @@ class Utils():
         _stochpy_.plt.xlim([10**-7,10**3])
         _stochpy_.plt.ylim([10**-9,10**3])
         
-    def doExample4(self):
-        """ Second Burstmodel example (available at http://stompy.sourceforge.net/examples.html) """
+    def DoExample4(self):
+        """ Second Burstmodel example (available at http://stochpy.sourceforge.net/examples.html) """
         import matplotlib.gridspec as gridspec
         sim_end = 100                
         mod.Model('Burstmodel.psc')
@@ -208,8 +209,8 @@ class Utils():
         _stochpy_.plt.yticks([0,20,40,60])
         _stochpy_.plt.text(101,27,'D',fontsize = 14)  
         
-    def doExample5(self):
-        """ Protein turnover example with and without cell division (available at http://stompy.sourceforge.net/examples.html) """
+    def DoExample5(self):
+        """ Protein turnover example with and without cell division (available at http://stochpy.sourceforge.net/examples.html) """
         smod = _stochpy_.SSA(File='CellDivision.psc',dir=_stochpy_.model_dir) # start SSA module with CellDivision model
         ### do protein synthesis without cell division ###
         smod.DoStochSim(end=10,mode='time')
